{% extends "kagi/base.html" %}
{% load i18n %}

{% block content %}
{{ block.super }}

<p>{% trans 'Scan this in your authenticator app:' %}</p>

<a href="{{ otpauth }}">
{{ qr_svg|safe }}
</a>

<p>
  {% trans "Or, if you can't scan a QR Code, enter this key as a time-based account:" %} <a href="{{ otpauth }}"><tt>{{ base32_key }}</tt></a>
</p>

<p>
  {% trans 'Then, enter the token it gives you.' %}
</p>

<form method="POST">
  {% csrf_token %}
  {{ form.as_p }}
  <button value="backup" name="type">{% trans 'Submit' %}</button>
</form>

{% endblock %}
