{% extends "kagi/base.html" %}
{% load i18n %}

{% block content %}
{{ block.super }}

<a href="{% url 'kagi:two-factor-settings' %}">{% trans '&larr; Back to settings' %}</a>

<ul>
  {% for code in object_list %}
  <li>{{ code.code }}</li>
  {% empty %}
  <li>{% trans 'You do not have any backup codes! Please create some!' %}</li>
  {% endfor %}
</ul>

<form method="POST">
  {% csrf_token %}
  <input type="submit" value="{% trans 'Create some backup codes' %}">
</form>

{% endblock %}
