{% extends "kagi/base.html" %}
{% load i18n %}

{% block content %}
{{ block.super }}
<h1>{% trans "Two Factor Settings" %}</h1>
<ul>
  <li><a href="{% url 'kagi:webauthn-keys' %}">{% trans 'Manage WebAuthn keys' %}</a></li>
  <li><a href="{% url 'kagi:backup-codes' %}">{% trans 'Manage backup codes' %}</a></li>
  <li><a href="{% url 'kagi:totp-devices' %}">{% trans 'Manage TOTP (Authenticator) devices' %}</a></li>
</ul>

<h2>{% trans "Status" %}</h2>
<ul>
  <li>{% trans "WebAuthn" %}: {% if webauthn_enabled %}{% trans "On" %}{% else %}{% trans "Off" %}{% endif %}</li>
  <li>{% trans "TOTP" %}: {% if totp_enabled %}{% trans "On" %}{% else %}{% trans "Off" %}{% endif %}</li>
  <li>{% trans "Backup codes" %}: {% if backup_codes_count %}{{ backup_codes_count }} {% trans "remaining" %}{% else %}{% trans "None generated" %}{% endif %}</li>
</ul>

{% endblock %}
