{% extends "kagi/base.html" %}
{% load i18n %}
{% load static %}

{% block content %}
{{ block.super }}
<p>{% trans 'To add a security key to your account, insert it, tap the button below, and accept the browser prompt to add the key.' %}</p>

<form id="register-form" name="register" method="get">
    {% csrf_token %}
    {{ form }}
    <button id="register" type="submit">{% trans 'Add WebAuthn Key' %}</button>
</form>

<script src="{% static 'kagi/webauthn.js' %}"></script>
{% endblock %}
